PROGRAM Fkeys;

{

   Program       : Fkeys.pas
   Author        : Bob Willoughby
   Date          : 05/10/90
   Revision      : 3.0
   Description   : Provides a means of setting DOS level function keys.
   Caveats       : F1, 3, and 9 should not be set because of possible
                    interference with DOS and 4DOS.
   Compiler      : Turbo Pascal 5.0

}

Uses
  crt, dos, turbo, arglist;

  Const

      SCCSID          = '@(#)fkeys.pas, 3.0, 05/10/90\n';

  Type

      MaxKeys         = 1..40;             { Total number of keys to program }
      StringLength    = String[79];
      ConfigKeys      = ARRAY [MaxKeys] of StringLength;  { Struct for keys }

  Var

    regs            :registers;

    ConfigFile      :File of ConfigKeys;  { File for saving keys }

    Found_Param,                          { Parameter found indicator }
    Change,                               { Key changed indicator }
    Quiet,                                { Redirection indicator }
    ResultIO        :Boolean;             { Result of file open test }

    Fkey_Str,
    Fname           :String[40];

    F_Dir           :Dirstr;
    F_Name          :Namestr;
    F_Ext           :Extstr;

    FkeyConfig      :ConfigKeys;

    Ansi_OK         :Byte;

    V_Mode,                               { Video Mode result }
    KeyHit,                               { Some other general types }
    Count2,
    Count           :Integer;

    SCCS_ID,
    Response2,
    Response        :StringType;

    Resp,
    Fkey            :Char;

{===========================================================================}


{   This procedure is used to display a help screen under cetain conditions   }

  Procedure PrintHelp;

    Begin
      ClrScr;
      Skipln;
      Riteln('This program allows the setting of all 40 DOS function keys.');
      Skipln;
      Riteln('Usage:  fkeys [ - or / ] [ irdcl? ] [ filename ] [q]');
      Skipln;
      Riteln('Where:    c - configure function keys');
      Riteln('          l - load function keys');
      Riteln('          r - reset function keys');
      Riteln('          d - display a saved file');
      Riteln('          i - initialize a file');
      Riteln('          ? - display this screen');
      Riteln('   filename - key configuration file');
      Riteln('              ( DEFAULT is FKEYS.CFG in FKEYS.EXE home directory)');
      Riteln('          q - quiet mode');
      Skipln;
      Riteln('NOTE: An Extended ANSI driver must be installed');
      Riteln('      for this program to work properly.');
      Riteln('      I suggest NANSI or FCONSOLE.');
      Skipln;
      Riteln('  Author   -  Bob Willoughby');
      Riteln('  Version  -  3.0');
      Riteln('  Compiler -  Turbo Pascal Ver. 5.0');
      Riteln('  Purpose  -  Provides means for setting DOS function keys')
    End;


{============================================================================}

{  Used to get the filename of the configuration file.  Defaults to fkeys.cfg  }

  Procedure GetFilename;

    Begin
      If (argc > 1) Then
        If (Upcase_Str(argv(2)) = '/Q') Then
           Fname := F_Dir+F_Name+'.CFG'      { assign default filename }
          Else
           Begin
             Fname := argv(2);       { or get filename from command line }
             If Pos('.', Fname) = 0 Then Fname := Fname+'.cfg';
           End
          Else
           Fname := F_Dir+F_Name+'.CFG';

      Assign(ConfigFile,Fname);       { Assign the file and open file }
      {$I-}
      Reset(ConfigFile);
      ResultIO := (IOResult = 0);
      {$I+}
      If not ResultIO Then                       { if not open now }
         If (Upcase_Str(argv(1)) = '/C') Then
            Rewrite(ConfigFile)                  { create the file }
         Else Begin
            If (Upcase_Str(argv(1)) <> '/I') Then Begin
               Writeln('Unable to open file ',Fname);    { or give error }
               Halt;
            End
          End
        Else
          If not EOF(ConfigFile) and (Upcase_Str(argv(1)) <> '/I') and
             (Upcase_Str(argv(1)) <> '/R') Then
             Read(ConfigFile, FkeyConfig)

    End;
{============================================================================}

{  This procedure resets the function keys to normal DOS status  }

PROCEDURE Reset_Functions;

Var
    Reset_Code    :String[81];

    Begin

      regs.DX := Ofs(Reset_Code) + 1;
      regs.DS := Seg(Reset_Code);
      regs.AH := $09;

      If argv(1) = '/C' Then
        Begin

          For Count := 1 to 10 Do Begin    { Initialize Config Table }
            str((Count+58):2, Fkey_Str);
            FkeyConfig[Count] := '[0;'+Fkey_Str+';0;'+Fkey_Str+';p'
          End;

          For Count := 11 to 40 Do Begin
            str((Count+73):2, Fkey_Str);
            FkeyConfig[Count] := '[0;'+Fkey_Str+';0;'+Fkey_Str+';p'
          End;
        End;

      For Count := 1 to 40 Do
        If FkeyConfig[Count] <> NULL Then
           Begin
                  { generate string to output }
             Reset_Code := Concat(ESC, FkeyConfig[Count], '$');

             intr($21, regs);

        End;

        Skipln;

        If (Quiet and (Upcase_Str(argv(1)) = '/R'))Then
          Riteln('Function keys reset')
    End;

{============================================================================}

{  This procedure reads the configuration file and loads the function keys  }

  Procedure SetFunctions;

    Var
        Key_Code       :String[81];     { String for setting up key }

    Begin

      Skipln;

      GetFileName;

      regs.DX := Ofs(Key_Code) + 1;
      regs.DS := Seg(Key_Code);
      regs.AH := $09;

      For Count := 1 to 40 Do
        If FkeyConfig[Count] <> NULL Then
           Begin
                 { generate string to output }
             Key_Code := Concat(ESC, FkeyConfig[Count], '$');

             intr($21, regs);

        End;

        If Quiet Then
          Writeln('Function keys set via ',Fname)  { tell user we're done }
    End;


{===========================================================================}


{  This procedure is used to configure the keys in the configuration file.  }

  Procedure ConfigureKeys;

    Begin

      GetFilename;

      Fkey := NULL;
                      { Configure keys until user quits }
      While ((Fkey <> 'x') and (Fkey <> 'X')) Do
        Begin
          ClrScr;
          Rite('Enter a Function Key to setup or "X" to quit : ');
          Fkey := Getkey;
          If not(Fkey IN['x','X',ESC]) Then Begin
             If (ord(Fkey) IN[187..196,212..241]) Then  { is it a function key }
               Begin
                 KeyHit := (ord(Fkey) - 186);    { convert the keycode }
                 Fkey := chr(ord(Fkey) - 128);
                 str(ord(Fkey):2, Fkey_Str);
                 Resp := 'y';
                 If KeyHit IN[1,3,9] Then     { test for F1, 3, or 9 }
                   Begin
                     Skipln;
                     Riteln('Changing this key can');
                     Riteln('cause SERIOUS interference with DOS . . .');
                     Skipln;
                     Riteln('Do you wish to continue ? (y/n)');
                     Resp := GetKey;
                     If Resp IN['y','Y'] Then      { Get function key to setup }
                       Begin
                         ClrScr;
                         Rite('Enter a Function Key to setup : ')
                       End
                   End;
                 If Resp IN['y','Y'] Then
                   Begin
                     If KeyHit > 10 then      { Is key normal function key }
                       KeyHit := KeyHit - 15; { or is it extended function key }

                     Case KeyHit of
                          { DISPLAY THE FUNCTION KEY TO SETUP }
                        1..10     : Writeln('F',KeyHit);
                       11..20     : Writeln('Shift F',KeyHit - 10);
                       21..30     : Writeln('Ctrl F',KeyHit - 20);
                       31..40     : Writeln('Alt F',KeyHit - 30);

                     End;  {EndCase}
                     Skipln;
                     Count2 := 7;

                     While ((FkeyConfig[KeyHit][Count2] <> '"') and
                            (Count2 < 12)) Do
                       Count2 := Count2 +1;

                     Count2 := Count2 + 1;

                     If Count2 <= 9 Then
                        While Count2 < 79 Do
                          If FkeyConfig[KeyHit][Count2] <> '"' Then
                           Begin
                             Write(FkeyConfig[KeyHit][Count2]);
                             Count2 := Count2 + 1
                            End Else
                             Count2 := 79;

                     Skipln;
                     Skipln;

                     Riteln('Enter the command you wish to perform ');
                     Riteln('- to Delete    ESC to leave unchanged ');

                     Skipln;

                     Response2 := GetString(Response, 66); { GET SETUP STRING }

                      { - deletes else store the new setup }

                     If (Response2 <> '-') and (Response2 <> ESC) Then
                       Begin
                         Change := TRUE;     { Set key changed }
                         FkeyConfig[KeyHit]:='[0;'+Fkey_Str+';"'+Response2+'";13p'
                       End
                      Else
                       If Response = '-' Then
                         Begin
                           Change := TRUE;  { set key changed and delete key }
                           For Count2 := 1 to 79 Do
                              FkeyConfig[KeyHit][Count2] := NULL;
                           FkeyConfig[KeyHit] := '[0;'+Fkey_Str+';0;'+Fkey_Str+';p'
                         End
                        Else
                         Response := ' ' { else do nothing and reset response }

                     End
               End;

          End;

      End;

                         { SAVE THE CONFIGURATION }

      If Change = TRUE Then Begin        { save only if change made }
         Resp := NULL;
         Skipln;
         Rite('Save Changes ? ');           { Save config to disk }
         Resp := Getkey;
         If Resp IN['Y','y'] Then Begin
            Rewrite(ConfigFile);
            Write(ConfigFile, FkeyConfig);
            Close(ConfigFile);
            Skipln;
            Writeln('Keys saved to ',Fname)
         End;
         Skipln;                         { setfunctions before exit }
         Rite('Set function keys now ? ');
         Resp := Getkey;
         Skipln;
         Skipln;
         If Resp IN['Y','y'] Then
           Begin
             SetFunctions
           End;

      End

  End;

{===========================================================================}

{  This procedure displays the function key settings.  }

  Procedure DispFunctions;

    Var   Count2        :Integer;

    Begin

      GetFileName;
      Skipln;
      Rite('Configuration file = ');
      Write(fname);
      Skipln;
                   { Search and display Funtion Keys }

      For Count := 1 to 40 Do
          If (FkeyConfig[Count][8] <> ';') and
             (FkeyConfig[Count][9] <> ';')  Then Begin
            Count2 := 1;
            While FkeyConfig[Count][Count2] <> '"' Do
              Count2 := Count2 + 1;
            Count2 := Count2 + 1;
            If FkeyConfig[Count][Count2] <> '"' Then
              Begin
                Skipln;
                Case Count of

                    1..10     : Write(' F',Count,' = ');
                   11..20     : Write('SF',Count - 10,' = ');
                   21..30     : Write('CF',Count - 20,' = ');
                   31..40     : Write('AF',Count - 30,' = ');

                End;
                          { Display Function Key Found }
              End;
            While Count2 < 79 Do
              If FkeyConfig[Count][Count2] <> '"' Then
                 Begin
                   Write(FkeyConfig[Count][Count2]);
                   Count2 := Count2 + 1
                 End
                Else
                 Count2 := 79
          End;

      Skipln;

    End;

{===========================================================================}


{  This procedure tests the paramaters and processes command accordingly  }

  Procedure ProcessParams(Param_Str: StringType);

  Begin

    For Count := 1 to 10 Do Begin              { Initialize Config Table }
      str((Count+58):2, Fkey_Str);
      FkeyConfig[Count] := '[0;'+Fkey_Str+';0;'+Fkey_Str+';p'
    End;

    For Count := 11 to 40 Do Begin
      str((Count+73):2, Fkey_Str);
      FkeyConfig[Count] := '[0;'+Fkey_Str+';0;'+Fkey_Str+';p'
    End;

    Change := FALSE;                           { Reset key changed indicator }

    Case Param_Str[2] of                       { What do we want to do }

       '?': Begin                              { HELP }
              Found_Param := TRUE;
              PrintHelp
            End;

       'I': Begin                              { INITIALIZE A FILE }

              Found_Param := TRUE;

              GetFileName;
              Skipln;
              If ResultIO Then Begin
                 Riteln('This will delete any previous information in');
                 Writeln('configuration file (',Fname,').');
                 Skipln;
                 Riteln('Are you sure you want to do this ?');
                 Resp := ' ';

                 Resp := Getkey;
               End Else
                 Resp := 'y';
              If Resp IN['y','Y'] Then Begin
                 Skipln;
                 Riteln('Do you wish to configure the keys ? ');
                 Resp := Getkey;
                 If Resp IN['y','Y'] Then
                    ConfigureKeys
                  Else Begin
                    ReWrite(ConfigFile);
                    Write(ConfigFile, FkeyConfig);
                    Skipln;
                    Writeln(Fname,' initialized.');
                 End
              End;
            End;

      'L': Begin                              { LOAD THE KEYS }
              Found_Param := TRUE;
              SetFunctions;
            End;


      'D': Begin                              { DISPLAY THE SETUP FILE }
              Found_Param := TRUE;
              DispFunctions;
            End;

      'R': Begin                             { RESET THE FUNCTION KEYS }
              Found_Param := TRUE;
              Reset_Functions;
            End;

      'C': Begin                              { CONFIGURE THE KEYS }
              Found_Param := TRUE;
              ConfigureKeys;
            End;

    End;

  End;

{===========================================================================}


  Begin                   {   Main Program Module   }

    SCCS_ID     := SCCSID;
    Quiet       := TRUE;
    Found_Param := FALSE;

    With regs Do Begin

      AX := $1A00;       { Check to see if MSDOS ANSI driver }
      intr($2F, regs);   { is installed }
      Ansi_OK := AL;     { store results here }
      AX := $0F00;       { Check on current }
      intr($10, regs);   { video mode }
      AH := 0;
      V_Mode := AX       { store that here }

    End;

    If Ansi_OK = $FF Then Begin    { cannot interface with DOS's ANSI }
       Riteln('fkeys:  error: unsupported ANSI driver');
       Halt
    End;

    If V_Mode IN[0..3] Then           { Check for snow if CGA }
       CheckSnow := TRUE;

    If argc > 1 Then
      If ((Upcase_Str(argv(2)) = '/Q') or (Upcase_Str(argv(3)) = '/Q')) Then
         Quiet := FALSE;

    If argc = 0 Then       { Knucklehead didn't tell me a what to do }
      Begin
        Riteln('This program allows setting of all 40 DOS function keys');
        Skipln;
        Riteln('Usage:  fkeys [ - or / ] [ irdcl? ] [ filename ] [q]');
        Riteln('       { fkeys -? for HELP }');
        Halt;
      End else begin
        Fsplit(argv(0),F_Dir,F_Name,F_Ext);
        ProcessParams(Upcase_Str(argv(1)))
      End;

      If (not Found_Param) Then Begin
         Writeln('fkeys:  error:  unknown option: ',argv(1));
         Riteln('Usage:  fkeys [ - or / ] [ irdcl? ] [ filename ] [q]');
         Riteln('       { fkeys -? for HELP }')
      End;
  End.
