{$D-}

UNIT turbo;

Interface

Uses
   Crt, Dos;

Type
   IntType       = Integer;
   PseudoString  = ARRAY[1..255] OF Char;
   StringType    = STRING[80];
   Str512        = ARRAY[0..511] OF Char;
   Str255        = STRING[255];
   Str80         = STRING[80];
   Str64         = STRING[64];
   Str10         = STRING[10];
   Str8          = STRING[8];
   Str2          = STRING[2];

Const

  { Define some important keystrokes: }

  NULL     =   #0;             Bell     =   #7;
  BS       =   #8;             LF       =  #10;
  CR       =  #13;             ESC      =  #27;

  HomeKey  = #199;             EndKey   = #207;  { cursor control keys }
  UpKey    = #200;             DownKey  = #208;
  PgUpKey  = #201;             PgDnKey  = #209;
  LeftKey  = #203;             InsKey   = #210;
  RightKey = #205;             DelKey   = #211;

  F1       = #187;             F6       = #192;  { function keys }
  F2       = #188;             F7       = #193;
  F3       = #189;             F8       = #194;
  F4       = #190;             F9       = #195;
  F5       = #191;             F10      = #196;
  SF1      = #212;             SF2      = #213;
  SF3      = #214;             SF4      = #215;
  SF5      = #216;             SF6      = #217;
  SF7      = #218;             SF8      = #219;
  SF9      = #220;             SF10     = #221;
  CF1      = #222;             CF2      = #223;
  CF3      = #224;             CF4      = #225;
  CF5      = #226;             CF6      = #227;
  CF7      = #228;             CF8      = #229;
  CF9      = #230;             CF10     = #231;
  AF1      = #232;             AF2      = #233;
  AF3      = #234;             AF4      = #235;
  AF5      = #236;             AF6      = #237;
  AF7      = #238;             AF8      = #239;
  AF9      = #240;             AF10     = #241;

  IntChar  =  #27;             SPC      = ' ';

  Procedure Skipln;
  Procedure RiteLn(xxx: StringType);
  Procedure Rite(xxx: StringType);
  Procedure ErrorHalt(Msg : StringType);
  Function MakeReal(VAR b) : Real;
  Function MakeInt(VAR b) : Integer;
  Function MakeStr(b : Byte) : String;
  Procedure Wait;
  Function GetKey : char;
  Function GetString ( Var   Response   :StringType;
                           MaxCount   :IntType ) : String;
  Function Upcase_Str(in_string: string) : string;
  Function Lower_Str(in_string: string) : string;

Implementation



{ **************************************************************************
*
* 	Program :  fakewrit.inc
*   Coded   :  Bob Willoughby
*   Purpose :  Shorten code generated by Write and Writeln statements
* 	Version :  1.0
*
*************************************************************************** }

Procedure Skipln;

 Begin Writeln; End;

Procedure RiteLn(xxx: StringType);

 Begin Writeln(xxx); End;

Procedure Rite(xxx: StringType);

 Begin Write(xxx); End;

{ **************************************************************************
*
*   Purpose :  Miscellaneous code
*   Author  :  J. Troutman
*
*************************************************************************** }

Procedure ErrorHalt(Msg : StringType);

  Begin
    Skipln;
    Riteln(Msg);
    Halt;
  End;

Function MakeReal(VAR b) : Real;
  Var
    r : ARRAY[1..4] OF Byte ABSOLUTE b;

  Begin
    MakeReal := (r[1]*1)+(r[2]*256)+(r[3]*65536.0)+(r[4]*16777216.0);
  End;

Function MakeInt(VAR b) : Integer;
  Var
    i : Integer ABSOLUTE b;

  Begin
    MakeInt := i;
  End;

Function MakeStr(b : Byte) : String;
  Var
    i : Integer;
    s : String;
  Begin
    i := b;
    Str(i:2, s);
    MakeStr := s;
  End;

Procedure Wait;
  Var
    c : Char;

  Begin
    Rite('Press any key to continue . . .');
    c := ReadKey;
  End;

(****************************************************************************
*
*   Function Name    : GetKey
*
*   Function Author  : Bob Willoughby
*                      ( With example from tutor.lib )
*
*   Description      : Reads a key in from the keyboard and determines
*                      if it is a normal or special key.
*                      If special, it adds 128 to the keyboard key
*                      to derive the proper key.
*
*****************************************************************************)

Function GetKey : char;

Var
    Ch        :char;

Begin

  Ch := ReadKey;

  If (Ch = NULL) and KeyPressed Then      { Check for extended character }

     Begin
		 Ch := ReadKey;
       Ch := Chr(Ord(Ch) + 128);    { change extended chars to upper ASCII }
     End;

  { Endif }

  GetKey := Ch;

End; { GetKey }

(*****************************************************************************
*
*   Function Name    : GetString
*
*   Function Author  : Bob Willoughby
*
*   Description      : This function when called with a string variable
*                      and string size will read the keyboard directly
*                      and build a string for the variable passed.
*
*                      For a list of accepted keys, consult the function
*                      and Turbo Pascal Users Guide.  Also consult the
*                      function GetKey.inc.
*
*****************************************************************************)

Function GetString ( Var   Response   :StringType;
                           MaxCount   :IntType ) : String;

  Var

      X,
      R,
      Count       : INTEGER;

      Ch          : CHAR;

  Begin

    Count := 1;

    While Count <= MaxCount Do
      Begin
        Ch := GetKey;

        If Ch IN['!'..'}',BS,ESC,CR,SPC] Then

             Case Ch of

                ESC     :Begin
                           Response[1] := ESC;
                           Response[0] := Chr(Ord(1));
                           Count := MaxCount + 1;
                         End;

                BS      :Begin
                           If Count <> 1 Then
                             Begin
                               GotoXY(WhereX - 1,WhereY);
                               Write(' ');
                               GotoXY(WhereX - 1,WhereY);
                               Count := Count - 1;
                             End
                         End;


               SPC,
            '!'..'}'     :Begin
                            Write(Ch);
                            Response[Count] := Ch;
                            Response[0] := Chr(Ord(MaxCount));
                            Count := Count + 1;
                          End;

                CR       :Begin
                            Response[0] := Chr(Ord(Count-1));
                            GetString := Response;
                            Exit;
                          End

             End;  { EndCase}


      End; { EndWhile }

      GetString := Response;

  End; { Function GetString }

(*****************************************************************************
*
*   Function Name    : Upcase_Str
*
*   Function Author  : Bob Willoughby
*
*   Description      : This function when called with a string variable
*                      will convert the variable to all uppercase characters
*                      and pass it back to the caller.
*
*****************************************************************************)

Function Upcase_Str(in_string: string) : string;

  Var

    out_string      :string;
    i               :integer;

  Begin
    For i := 1 to length(in_string) Do Begin
      out_string[i] := upcase(in_string[i])
    End;
    out_string[0] := chr(i);
    Upcase_Str := out_string;

  End;

(*****************************************************************************
*
*   Function Name    : Lower_Str
*
*   Function Author  : Bob Willoughby
*
*   Description      : This function when called with a string variable
*                      will convert the variable to all lowercase characters
*                      and pass it back to the caller.
*
*****************************************************************************)

Function Lower_Str(in_string: string) : string;

  Var

    out_string      :string;
    i,j             :integer;

  Begin
    For i := 1 to length(in_string) Do Begin
       If in_string[i] IN['A'..'Z'] Then Begin
          j := ord(in_string[i]);
          j := j + 32;
          out_string[i] := chr(j);
       End
      Else
       out_string[i] := in_string[i]
    End;
    out_string[0] := chr(i);
    Lower_Str := out_string;

  End;


END.
